<?php

namespace Drupal\cms_content_sync\Plugin\cms_content_sync\field_handler;

use Drupal\cms_content_sync\PullIntent;
use Drupal\cms_content_sync\PushIntent;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\user\Entity\User;

/**
 * Providing a minimalistic implementation for any field type.
 *
 * @FieldHandler(
 *   id = "cms_content_sync_default_user_reference_handler",
 *   label = @Translation("Default User Reference"),
 *   weight = 80
 * )
 *
 * @package Drupal\cms_content_sync\Plugin\cms_content_sync\field_handler
 */
class DefaultUserReferenceHandler extends DefaultEntityReferenceHandler {

  const IDENTIFICATION_NAME = 'name';
  const IDENTIFICATION_EMAIL = 'mail';
  const IDENTIFICATION_SYNC_USER = 'sync_user';

  /**
   * {@inheritdoc}
   */
  public static function supports($entity_type, $bundle, $field_name, FieldDefinitionInterface $field) {
    if (!in_array($field->getType(), ["entity_reference", "entity_reference_revisions"])) {
      return FALSE;
    }

    $type = $field->getSetting('target_type');
    return $type == 'user';
  }

  /**
   * @inheritdoc
   */
  protected function forceMergeOverwrite() {
    return $this->fieldName == 'revision_uid';
  }

  /**
   * @inheritdoc
   */
  public function getHandlerSettings($current_values, $type = 'both') {
    $options = [
      'identification' => [
        '#type' => 'select',
        '#title' => 'Identification',
        '#options' => [
          self::IDENTIFICATION_EMAIL => 'Mail',
          self::IDENTIFICATION_NAME => 'Name',
          self::IDENTIFICATION_SYNC_USER => 'Sync User',
        ],
        '#default_value' => $current_values['identification'] ?? self::IDENTIFICATION_EMAIL,
      ],
    ];

    return $options;
  }

  /**
   * @inheritdoc
   */
  protected function loadReferencedEntity(PullIntent $intent, $definition) {
    $property = $this->settings['handler_settings']['identification'];

    if ($property == self::IDENTIFICATION_SYNC_USER) {
      $uid = \Drupal::service('keyvalue.database')->get('cms_content_sync_user')->get('uid');

      return User::load($uid);
    }
    else {
      if (empty($definition[$property])) {
        return NULL;
      }

      /**
       * @var \Drupal\user\Entity\User[] $entities
       */
      $entities = \Drupal::entityTypeManager()
        ->getStorage('user')
        ->loadByProperties([$property => $definition[$property]]);

      return reset($entities);
    }
  }

  /**
   * @inheritdoc
   */
  protected function serializeReference(PushIntent $intent, EntityInterface $reference, $value) {
    return [
      self::IDENTIFICATION_EMAIL => $reference->get('mail')->value,
      self::IDENTIFICATION_NAME  => $reference->get('name')->value,
    ];
  }

}
